/*
  Persistence of Vision Ray Tracer Scene Description File
  
  Stacked heightfield-coins demo
  
  --
  Jaime Vives Piqueres, 2003, revised May 2007.

*/

// *** trace control ***
#declare use_area=0;
#declare use_rad=0;  // radiosity: 0=no rad, 1=load, 2=save

// *** global setings **
global_settings{
 assumed_gamma 1.0
 #if (use_rad)
 radiosity{  
  #if (use_rad=2)
   count 50
   nearest_count 10
   error_bound .5
   recursion_limit 1
   save_file "coins.rad"
  #else
   pretrace_start 1 pretrace_end 1
   error_bound .5
   recursion_limit 1
   always_sample off
   load_file "coins.rad"
  #end
 }
 #end
}
#default{texture{finish{ambient 0}}}

// *** standard includes ***
#include "colors.inc"
#include "textures.inc"
#include "metals.inc"

// *** room ***
box{-.5,.5 hollow
 scale <500,250,500>
 pigment{White}
 translate <0,125,0>
}

// *** table ***
#declare mesa=
box{-.5,.5 scale <100,.05,100>
 pigment{HuntersGreen}
 normal{quilted scale .1} 
 translate 80*y
}
object{mesa}

// *** coins ***
// metal normal bump map
#declare n_scratchs_base=
normal{
 bump_map{png "hf_scratch2"}
}
#declare n_metal=
normal{
 wrinkles turbulence .1
 normal_map{
  [0 n_scratchs_base]
  [1 n_scratchs_base translate .5 scale .5]
 }
 scale 1
 rotate 90*x
}
// metal finish
#declare Metal=finish{Metal ambient 0 reflection{.1,.9}}
// coin macro
#include "i_coin.inc"
// create coin heightfields
#declare hf_5pu= 
height_field{
 png "coin-map1"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_5pu_back=
height_field{
 png "coin-map6"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_10pu=
height_field{
 png "coin-map2"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_10pu_back=
height_field{
 png "coin-map5"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_50pu=
height_field{
 png "coin-map3"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_50pu_back=
height_field{
 png "coin-map4"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_100pu=
height_field{
 png "coin-map7"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}
#declare hf_100pu_back=
height_field{
 png "coin-map8"
 water_level 0.4
 translate -.5
 scale <1.05,.5,1.05>
 translate .4*y
}

// *** join coin pieces ***
// - 5 pov units coin -
#declare ch5=.22;
#declare coin5pu=object{coin(1.1,ch5,Copper,n_metal,Metal,hf_5pu,hf_5pu_back)}
// - 10 pov units coin -
#declare ch10=.24;
#declare coin10pu=object{coin(1.2,ch10,Gold,n_metal,Metal,hf_10pu,hf_10pu_back)}
// - 50 pov units -
#declare ch50=.28;
#declare coin50pu=object{coin(1.5,ch50,Brass,n_metal,Metal,hf_50pu,hf_50pu_back)}
// - 100 pov units -
#declare ch100=.3;
#declare coin100pu=object{coin(1.7,ch100,Silver,n_metal,Metal,hf_100pu,hf_100pu_back)}

// *** create the coin stacks ***
#declare prueba=object{mesa}
#declare rc=seed(35);
#declare ma=<10,0,10>;
#declare nc=10;
#declare ic=0;
union{
 #while (ic<nc)
  #declare Start=<-ma.x*.5+ma.x*rand(rc),100,-ma.z*.5+ma.z*rand(rc)>;
  #declare Norm=<0,0,0>;
  #declare Inter=trace(prueba,Start,-1*y,Norm);
  #if (Inter.y<81.1)
   #if (rand(rc)<.25)
    object{coins_stack(coin5pu,ch5,int(4+14*rand(rc)),1000*rand(rc)) translate Inter+ch5*.5*y}
    #declare prueba=union{object{prueba} cylinder{<0,0,0>,<0,1,0>,1.1*2.2 translate Inter+1*y}}
   #else
    #if (rand(rc)<.333)
     object{coins_stack(coin10pu,ch10,int(4+20*rand(rc)),1000*rand(rc)) translate Inter+ch10*.5*y}
     #declare prueba=union{object{prueba} cylinder{<0,0,0>,<0,1,0>,1.2*2.2 translate Inter+1*y}}
    #else
     #if (rand(rc)<.5)
      object{coins_stack(coin50pu,ch50,int(5+15*rand(rc)),1000*rand(rc)) translate Inter+ch50*.5*y}
      #declare prueba=union{object{prueba} cylinder{<0,0,0>,<0,1,0>,1.5*2.2 translate Inter+1*y}}
     #else
      object{coins_stack(coin100pu,ch100,int(4+16*rand(rc)),1000*rand(rc)) translate Inter+ch100*.5*y}
      #declare prueba=union{object{prueba} cylinder{<0,0,0>,<0,1,0>,1.7*2.2 translate Inter+1*y}}
     #end
    #end
   #end
   #declare ic=ic+1;
  #end
 #end
}

// *** lights ***
#declare rl=seed(122);
// incandescent lamp
#declare lamp=
light_source{
 0,(Gold+White*2)*1600
 #if (use_area)
 area_light 9*x,9*z,4*use_area,4*use_area jitter orient
 #end
 fade_distance 1
 fade_power 2
}
object{lamp translate <-100+200*rand(rl),100+100*rand(rl),-100+200*rand(rl)>}
// fluorescent tubes
#declare fluorescent=
light_source{
 0,(YellowGreen+White*2)*4800
 #if (use_area)
 area_light 9*x,9*z,4*use_area,4*use_area jitter orient
 #end
 fade_distance 1
 fade_power 2
}
object{fluorescent translate <-200+400*rand(rl),219,-200+400*rand(rl)>}

// *** camera ***
#declare r_cam=seed(14);
#declare f=32;
#declare fp=<0,81,0>;
#declare cl=<0,81+50*rand(r_cam),-50*rand(r_cam)>;
camera{
 location cl
 direction 3.5*z
 look_at fp
 rotate 360*rand(r_cam)*y
}
